@@ -75,6 +75,8 @@ _S(AUDIT_DAEMON_CONFIG,              "DAEMON_CONFIG"                 )
 //_S(AUDIT_DAEMON_RECONFIG,          "DAEMON_RECONFIG"               )
 _S(AUDIT_DAEMON_ROTATE,              "DAEMON_ROTATE"                 )
 _S(AUDIT_DAEMON_RESUME,              "DAEMON_RESUME"                 )
+_S(AUDIT_DAEMON_ACCEPT,              "DAEMON_ACCEPT"                 )
+_S(AUDIT_DAEMON_CLOSE,               "DAEMON_CLOSE"                  )
 _S(AUDIT_SYSCALL,                    "SYSCALL"                       )
 //_S(AUDIT_FS_WATCH,                 "FS_WATCH"                      )
 _S(AUDIT_PATH,                       "PATH"                          )
@@ -95,6 +97,8 @@ _S(AUDIT_FD_PAIR,                    "FD_PAIR"                       )
 _S(AUDIT_OBJ_PID,                    "OBJ_PID"                       )
 _S(AUDIT_TTY,                        "TTY"                           )
 _S(AUDIT_EOE,                        "EOE"                           )
+_S(AUDIT_BPRM_FCAPS,                 "BPRM_FCAPS"                    )
+_S(AUDIT_CAPSET,                     "CAPSET"                        )
 _S(AUDIT_AVC,                        "AVC"                           )
 _S(AUDIT_SELINUX_ERR,                "SELINUX_ERR"                   )
 _S(AUDIT_AVC_PATH,                   "AVC_PATH"                      )
@@ -110,6 +114,9 @@ _S(AUDIT_MAC_IPSEC_ADDSA,            "MAC_IPSEC_ADDSA"               )
 _S(AUDIT_MAC_IPSEC_DELSA,            "MAC_IPSEC_DELSA"               )
 _S(AUDIT_MAC_IPSEC_ADDSPD,           "MAC_IPSEC_ADDSPD"              )
 _S(AUDIT_MAC_IPSEC_DELSPD,           "MAC_IPSEC_DELSPD"              )
+_S(AUDIT_MAC_IPSEC_EVENT,            "MAC_IPSEC_EVENT"               )
+_S(AUDIT_MAC_UNLBL_STCADD,           "MAC_UNLBL_STCADD"              )
+_S(AUDIT_MAC_UNLBL_STCDEL,           "MAC_UNLBL_STCDEL"              )
 _S(AUDIT_ANOM_PROMISCUOUS,           "ANOM_PROMISCUOUS"              )
 _S(AUDIT_ANOM_ABEND,                 "ANOM_ABEND"                    )
 #ifdef WITH_APPARMOR
@@ -139,6 +146,7 @@ _S(AUDIT_ANOM_MK_EXEC,               "ANOM_MK_EXEC"                  )
 _S(AUDIT_ANOM_ADD_ACCT,              "ANOM_ADD_ACCT"                 )
 _S(AUDIT_ANOM_DEL_ACCT,              "ANOM_DEL_ACCT"                 )
 _S(AUDIT_ANOM_MOD_ACCT,              "ANOM_MOD_ACCT"                 )
+_S(AUDIT_ANOM_ROOT_TRANS,            "ANOM_ROOT_TRANS"               )
 _S(AUDIT_RESP_ANOMALY,               "RESP_ANOMALY"                  )
 _S(AUDIT_RESP_ALERT,                 "RESP_ALERT"                    )
 _S(AUDIT_RESP_KILL_PROC,             "RESP_KILL_PROC"                )
